#!/usr/bin/perl
use strict;
use Getopt::Std;
use File::Basename;

my($version) = "0.4.0";

my($options) = "PTextSplit [options] <file>
   Author: Raymond Choc, raymond.choc(at)gmx.net
   split polish text files separated by type and create csv-file for POI_db_creator.
   Options:
     -v version info
     -q be quiet
     -h print this help
     -d destination file name (or path), usefull if you read more than one file
        and write them to a multi-track-format or a different directory \n";

my($bequiet) = 0;
my($ifname,$ofname,$destFilename);

die("Option error!\n$options")
    unless getopts('vqhd:');

if (defined($Getopt::Std::opt_v)) {
    print STDERR "gpsconv Version $version\n";
    exit;
}if (defined($Getopt::Std::opt_h)) {
  print STDERR ($options);
  exit 0;
}
if (defined($Getopt::Std::opt_d)) {
	$destFilename = $Getopt::Std::opt_d;
}

if (defined($Getopt::Std::opt_q)) {
	$bequiet = 1;
}

if (!defined($ARGV[0])) {
	die("No file to split!\n$options");
}

$ifname = $ARGV[0];

#my(%ext);
#%ext = (nmea=>"nmea",exp=>"log", mag=> "txt", ovl=>"ovl", tk=>"tk", mm=>"txt", fug=>"txt", 
#				pcxg=>"trk", pcxm=>"trk", garm=>"txt", g7t=>"g7t", kml=>"kml", gpx=>"gpx");

my(%typMap) = &readConfig;

#foreach (keys %typMap){
#	foreach (keys %{$typMap{$_}}){
#		print $_."\n";
#	}
#}

if (!$bequiet) {
	print STDERR "Reading data from '$ifname' ...\n";
}
my(%mpMap)=readPText($ifname,%typMap);
if (!$bequiet) {
	print STDERR "Writing data to csv-file ...\n";
}
writeCSV($destFilename,%{$mpMap{CSV}});
delete $mpMap{CSV};
if (!$bequiet) {
	print STDERR "Writing data to separate polish-text-file ...\n";
}
writePText($destFilename,%mpMap);

#foreach (keys %mpMap){
#	foreach my $typ (keys %{$mpMap{$_}}){
#		print #@{$mpMap{$_}{$typ}->{data}}."\n";
#	}
#}

# END MAIN #

######### Subroutines ##########################

sub readConfig{
	unless (open(CFG,"<maping.def")) {
	    die "Cannot open 'maging.def' for reading";
	}
	my(%typMap);
	my($csvMode)=0;
	my($category);
	my($name);
	while(<CFG>){
		chomp;
  	if (/^\s*$/ || /^(\#)/ ) {
			next;
  	}
  	if(/\[SPLIT\]/){
  		$csvMode=0;
  		next;
  	}elsif(/\[CSV\]/){
  		$csvMode=1;
  		next;
  	}
  	if(!$csvMode){
  		if(/^\[(POLYLINE|POLYGON|POI)\]/){
  			$category=$1;
  			next;
  		}else{
  			/^0x([0-9a-f]+),\s+(\w+)/;
				$_ = $1;$name=$2;
				if($_ ne ""){
					$_ =~ s/^0+//; 
					$_ = $_ eq ""?0:$_;
					s/^0+//g; 
  				$typMap{$category}{'0x'.$_}={cat=>$category, name=>$name};
#					print "S ".'0x'.$1."=".$typMap{'0x'.$1}->{cat}.",".$typMap{'0x'.$1}->{name}."\n";
				}
  		}
  	}
  	else{
			/^0x([0-9a-f]+),\s+(\w+)/;
			$_ = $1;$name=$2;
			if($_ ne ""){
				$_ =~ s/^0+//; 
				$_ = $_ eq ""?0:$_;
				$typMap{CSV}{'0x'.$_}={cat=>"csv", name=>$name};
	#			print "CSV ".'0x'.$1."=".$typMap{'0x'.$1}->{cat}.",".$typMap{'0x'.$1}->{name}."\n";
			}
  	}
	}
	close CFG;
	%typMap;
}

sub readPText{
	my($ifname,%typMap) =@_;
	my(%mpMap);
	unless (open(INPUT,"<$ifname")) {
	    die "Cannot open '$ifname' for reading";
	}

	my($read)=0;
	my($readHeader)=0;
	my($cat,$type,@arr,$label,$data4,%data);
	while (<INPUT>){
		chomp;
  	if (/^\s*$/){
			next;
  	}
#  	print $_."\n";
		if(/^\[(IMG ID)\]/){
			$cat=$1;
			$type="header";
			$readHeader =1;
		}elsif(/^\[(POLYLINE|POLYGON|POI)\]/){
 			$cat=$1;
		}elsif(/^Type=0x([0-9a-f]+)/){
			$type='0x'.$1;
			if($cat eq "POI" && defined $typMap{CSV}{$type}){
				$cat="CSV";
				$type=$typMap{CSV}{$type}->{name};
#				$read=1;
			}
			%data={};
			$read=1;
		}elsif(/^\[END\]/ || /^\[END-IMG ID\]/){
			$read=0;
			$readHeader =0;
			my(%tmp)=%data;
#			if($type eq "header"){
#				print $data{header}."\n";
#			}
			if(defined $mpMap{$cat}{$type}){
				push @{$mpMap{$cat}{$type}}, \%tmp;
			}else{
				my(@arr);
				push @arr, \%tmp;				
				$mpMap{$cat}{$type}=\@arr;
			}
#			if($cat eq "CSV"){
#				@arr = @{$mpMap{$cat}{$type}};
#				%data=%{$arr[$#arr]};
#				print "type=".$type."\n";
#				print"label=".$data{label}."\n";
#				print"data=".$data{data}."\n";
#				last;
#			}
			next;
		}elsif($read){
			if(/^Label=(.*)/){
				$data{label}=$1;
#				print "L ".$data{label}."\n";
			}elsif(/^Data(\d+)=(.*)/){
#				print "D ".$2."\n";
				$data{data}=$2;
				$data{level}=$1;
			}
		}elsif($readHeader){
			$data{header}.=$_."\n";
		}
	}
	%mpMap;
}

sub writeCSV{
	my($destFilename,%csvMap) = @_;
#	my($ofname) = $destFilename ne ""?$destFilename.".csv.txt":$ifname.".csv.txt";
	my($ofname);
	if(defined $destFilename){
		my($body,$path,$suffix) = fileparse($destFilename,qw{\.\w*$});
		$ofname = $path;
		if($body ne ""){
			$ofname .= $body;
		}else{
			my($body,$path,$suffix) = fileparse($ifname,qw{\.\w*$});
			$ofname .=$body;
		}
	}else{
		my($body,$path,$suffix) = fileparse($ifname,qw{\.\w*$});
		$ofname = $path.$body;
	}
	$ofname .= ".csv.txt";
	
	unless (open(OUT,">$ofname")) {
		die "Cannot open '$ofname' for writing";
	}
	my($type,@arr,%data,$text,$i);
	foreach $type (sort keys %csvMap){
		printf OUT "\n[%s]\n", $type;
		@arr = @{$csvMap{$type}};
		for($i=0;$i<=$#arr;++$i){
			%data=%{$arr[$i]};
#			print "type=".$type."\n";
#			print"label=".$data{label}."\n";
#			print"data=".$data{data}."\n";
#		foreach (@arr){
#			%data = %{$_};
			$text = $data{data};
			$text =~ /\(([\d\.\-]*),([\d\.\-]*)\)/;
#			$text =~ s/\)//g;
#			$text =~ s/,/, /g;
			$text = $2.", ".$1;
			if(!defined $data{label}){
				$data{label}=$type;
#				print $data{label}."\n";
			} 
			printf OUT "%s, %s\n", $data{label}, $text;	
		} 
	}
	close OUT;

}

sub writePText{
	my($destFilename,%mpMap) = @_;
	
	my($cat,$type);
	my(@arr,%data,$text,$i);
	my($ofname);
	foreach $cat (keys %mpMap){
		if($cat eq "IMG ID"){
			next;
		}
#		print "cat=$cat\n";
		foreach $type (keys %{$mpMap{$cat}}){
#			print "TYPE=$type\n";
			$ofname = destFilename($cat,$type,$destFilename,%typMap);
#			print "FILE=$ofname\n";
			unless (open(OUT,">$ofname")) {
				die "Cannot open '$ofname' for writing";
			}
			printf OUT "; Generated by PTextSplit $version\n";
			printf OUT "\n"
								."[IMG ID]\n";
			@arr = @{$mpMap{"IMG ID"}{"header"}};
			%data = %{$arr[0]};
			printf OUT "%s", $data{header};
			printf OUT "[END-IMG ID]"
								."\n";
			@arr = @{$mpMap{$cat}{$type}};
			for($i=0;$i<=$#arr;++$i){
				%data=%{$arr[$i]};
				printf OUT "\n[%s]\n", $cat;
				printf OUT "Type=%s\n", $type;
				if($data{label} ne ""){
					printf OUT "Label=%s\n", $data{label};
				}
				printf OUT "Data%d=%s\n", $data{level}, $data{data};
				printf OUT "[END]\n";
			}
			close OUT;
		}
	}
}

sub destFilename{
	my($cat,$type,$destFileName,%typMap) = @_;
	my($ofname);
	if(defined $destFilename){
		my($body,$path,$suffix) = fileparse($destFileName,qw{\.\w*$});
		$ofname = $path.$cat."_";
		if($body ne ""){
			$ofname .= $body;
		}else{
			my($body,$path,$suffix) = fileparse($ifname,qw{\.\w*$});
			$ofname .=$body;
		}
	}else{
		my($body,$path,$suffix) = fileparse($ifname,qw{\.\w*$});
		$ofname = $path.$cat."_".$body;
	}
	if(exists $typMap{$cat}{$type}){ 
		$ofname .= "_".$typMap{$cat}{$type}->{name};
	}
	$ofname .=".".$type.".mp";
	$ofname;
}


